/* BFD back-end for Borland TDS debugging information.

   Contributed by Troy Rollo - http://www.troy.rollo.com/
   Copyright 2000 Free Software Foundation, Inc.

This file is part of BFD, the Binary File Descriptor library.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#include "bfd.h"
#include "sysdep.h"
#include "libbfd.h"
#include "tds.h"


const struct bfd_target *
borland_tds_object_p (abfd)
     bfd *abfd;
{
  tds_file_header header;
  unsigned long section_start;
  struct stat statbuf;

  if (!abfd->opened_for_symbols
      || bfd_stat (abfd, &statbuf) != 0
      || statbuf.st_size < sizeof(header)
      || bfd_seek (abfd, statbuf.st_size - sizeof(header), SEEK_SET) != 0
      || bfd_read (&header, sizeof(header), 1, abfd) != sizeof(header))
    return 0;
  if (TDS_VALID_HEADER (header))
    {
      if (header.tfh_offset.tfh_start_of_debug_info > (unsigned long) statbuf.st_size)
	return 0;
      section_start = statbuf.st_size - header.tfh_offset.tfh_start_of_debug_info;
    }
  else
    {
      section_start = 0;
    }
  if (bfd_seek (abfd, section_start, SEEK_SET) != 0
      || bfd_read (&header, sizeof (header), 1, abfd) != sizeof (header) ||
      !TDS_VALID_HEADER (header))
    return 0;

  abfd->flags |= HAS_LINENO | HAS_DEBUG | HAS_SYMS | HAS_LOCALS;

  bfd_get_symcount (abfd) = 0; /* FIXME */

  bfd_get_start_address (abfd) = section_start;

  return abfd->xvec;
}

long
borland_tds_get_symtab_upper_bound (abfd)
     bfd *abfd __attribute__((unused));
{
  return 0;
}

long
borland_tds_get_symtab (abfd, symtab)
     bfd *abfd __attribute__((unused));
     struct symbol_cache_entry **symtab __attribute__((unused));
{
  return 0;
}

struct symbol_cache_entry *
borland_tds_make_empty_symbol (abfd)
     bfd *abfd __attribute__((unused));
{
  return 0;
}

void
borland_tds_print_symbol (abfd, file, symbol, how)
     bfd *abfd __attribute__((unused));
     PTR file __attribute__((unused));
     struct symbol_cache_entry *symbol __attribute__((unused));
     bfd_print_symbol_type how __attribute__((unused));
{
  return;
}

void
borland_tds_get_symbol_info (abfd, symbol, info)
     bfd *abfd __attribute__((unused));
     struct symbol_cache_entry *symbol __attribute__((unused));
     symbol_info *info __attribute__((unused));
{
  return;
}

boolean
borland_tds_bfd_is_local_label_name (abfd, symbol)
     bfd *abfd __attribute__((unused));
     const char *symbol __attribute__((unused));
{
  return false;
}

alent *
borland_tds_get_lineno (abfd, symbol)
     bfd *abfd __attribute__((unused));
     struct symbol_cache_entry *symbol __attribute__((unused));
{
  return 0;
}

boolean
borland_tds_find_nearest_line (abfd, section, symbols, offset, file, func, line)
     bfd *abfd __attribute__((unused));
     struct sec *section __attribute__((unused));
     struct symbol_cache_entry **symbols __attribute__((unused));
     bfd_vma offset __attribute__((unused));
     CONST char **file __attribute__((unused));
     CONST char **func __attribute__((unused));
     unsigned int *line __attribute__((unused));
{
  return false;
}

asymbol *
borland_tds_bfd_make_debug_symbol (abfd, ptr, size)
     bfd *abfd __attribute__((unused));
     void *ptr __attribute__((unused));
     unsigned long size __attribute__((unused));
{
  return 0;
}

long
borland_tds_read_minisymbols (abfd, dynamic, minisymsp, sizep)
     bfd *abfd __attribute__((unused));
     boolean dynamic __attribute__((unused));
     PTR *minisymsp __attribute__((unused));
     unsigned int *sizep __attribute__((unused));
{
  return 0;
}

asymbol *
borland_tds_minisymbol_to_symbol (abfd, dynamic, minisym, sym)
     bfd *abfd __attribute__((unused));
     boolean dynamic __attribute__((unused));
     const PTR minisym __attribute__((unused));
     asymbol *sym __attribute__((unused));
{
  return 0;
}

bfd_target borland_tds_vec =
{
  "borland-tds",			/* name */
  bfd_target_borland_flavour,		/* flavour */
  BFD_ENDIAN_LITTLE,			/* byteorder */
  BFD_ENDIAN_LITTLE,			/* header_byteorder */
  HAS_LINENO
    | HAS_DEBUG
    | HAS_SYMS
    | HAS_LOCALS
    /* | HAS_RELOC???*/,		/* object_flags */
  SEC_HAS_CONTENTS
    | SEC_DEBUGGING,			/* section_flags */
  '_',				/* symbol_leading_char */
  '\0',					/* ar_pad_char */
  0,					/* ar_max_namelen */

  bfd_getl64, bfd_getl_signed_64, bfd_putl64,
     bfd_getl32, bfd_getl_signed_32, bfd_putl32,
     bfd_getl16, bfd_getl_signed_16, bfd_putl16, /* data */
  bfd_getl64, bfd_getl_signed_64, bfd_putl64,
     bfd_getl32, bfd_getl_signed_32, bfd_putl32,
     bfd_getl16, bfd_getl_signed_16, bfd_putl16, /* hdrs */

  {_bfd_dummy_target, borland_tds_object_p,
     _bfd_dummy_target, _bfd_dummy_target},	/* _bfd_check_format */
  {bfd_false, bfd_false, bfd_false, bfd_false},	/* _bfd_set_format */
  {bfd_false, bfd_false, bfd_false, bfd_false}, /* _bfd_write_contents */

  BFD_JUMP_TABLE_GENERIC (_bfd_generic),
  BFD_JUMP_TABLE_COPY (_bfd_generic),
  BFD_JUMP_TABLE_CORE (_bfd_nocore),
  BFD_JUMP_TABLE_ARCHIVE (_bfd_noarchive),
  BFD_JUMP_TABLE_SYMBOLS (borland_tds),
  BFD_JUMP_TABLE_RELOCS (_bfd_norelocs),
  BFD_JUMP_TABLE_WRITE (_bfd_nowrite),
  BFD_JUMP_TABLE_LINK (_bfd_nolink),
  BFD_JUMP_TABLE_DYNAMIC (_bfd_nodynamic),

  NULL,

  NULL

};

